/*
	
	PsiNCPWindow.cpp
	
	Programmstart: 12.9.1998 (Michael Pieper)
	
	Diese Funktion stellt die komplette Handhabung des Statusfensters dar.
	Die verschiedenen Funktionen werden über TabViews geregelt!
*/

// nur zum debuggen!=======================
#include "../DebugWin/debug.h"
//=========================================
#include <SerialPort.h>
#include "thread.h"
#include "PsiNCPWindow.h"
#include "PsionNCP.h"
#include "ReadSerial.h"
#include "WriteSerial.h"

PsiNCPWindow::PsiNCPWindow(BRect frame)
				: BWindow(frame, "Psion-BeOS", B_TITLED_WINDOW, B_NOT_ZOOMABLE )
{
	BRect		r1, r2, all;
	BView		*backgrnd;
	BTabView	*tabView; 
	float		h, w;
	int32		w1, w2;
	
	// Setzen wir die Einstellungen auf den Standardwert!
	
	*PortName = '\0';
	DataRate = (data_rate) 0;
	connected = false;
	Port = new BSerialPort();
	LoadPreferences();
	
	// Zuerst brauchen wir mal einen View, der in unserem Fenster liegt und als Hintergrund
	// dient. Mehr braucht das Teil nicht zu können, daher wird der View ohne Ableitung
	// direkt eingebunden!
	
	all = Bounds();
	backgrnd = new BView(all, "backgrnd", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	backgrnd->SetViewColor(216,216,216,0);
	AddChild(backgrnd);
	
	// An diesen View hängen wir zwei Buttons an. Links Disconnect und rechts Connect.
	
	connect = new BButton(all, NULL, "Connect", new BMessage(MSG_CONNECT), B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	connect->ResizeToPreferred();
	r1 = connect->Frame();
	
	discon = new BButton(all, NULL, "Disconnect", new BMessage(MSG_DISCONNECT), B_FOLLOW_BOTTOM | B_FOLLOW_LEFT);
	discon->ResizeToPreferred();
	r2 = discon->Frame();
	
	// Und zwar sollen diese Buttons am unteren Ende des Views landen!
	// Dazu muß ein bißchen was berechnet werden!

	h = all.bottom - (r1.Height() + 5);
	w1 = r1.IntegerWidth();
	w2 = r2.IntegerWidth();
	w = (all.right - (w1 + w2 + 10)) / 2;
	
	// Jetzt können wir die Buttons dahin schieben, bevor sie "Geaddet" werden.
	
	connect->MoveTo(all.right - (w1+w), h);
	discon->MoveBy(w, h);
	
	backgrnd->AddChild( connect );
	backgrnd->AddChild( discon );

	connect->MakeDefault(true);
	discon->SetEnabled(false);
	
	if (DataRate != 0 && *PortName != '\0') connect->SetEnabled(true);
	else connect->SetEnabled(false);

	// Als nächstes soll darüber ein BTabView eingebaut werden.
	r1 = Bounds();
	r1.InsetBy(5,5); 
	r1.bottom -= r2.Height() + 10;
	
	tabView = new BTabView(r1, "tab_view"); 
	tabView->SetViewColor(216,216,216,0); 
   
	r1 = tabView->Bounds(); 
	r1.InsetBy(5,5); 
	r1.bottom -= tabView->TabHeight(); 
	tabView->AddTab(new PsiNCPView(r1, "State", "ein Text"), NULL); 
	tabView->AddTab(new SerialView(r1, "Settings", Port, PortName, DataRate), NULL);

	backgrnd->AddChild(tabView);
//	debug_box = (BCheckBox *) FindView("WriteDebugFile");
//	printf("debug_box= %ld\n", debug_box);
//	if (debug) debug_box->SetValue(1);
}

// Wird das Fenster wirklich gelöscht, geben wir wieder alles frei!

PsiNCPWindow::~PsiNCPWindow( void ) {

	delete Port;

}

// Dieser Programmteil empfängt alle Nachrichten des Fensters
void PsiNCPWindow::MessageReceived(BMessage *msg) {
	
	char		*str, nam[2]="0";
	int8		nr;
	thread_id	thrid;
	
	switch(msg->what) {
		case MSG_B_9600_BPS:	DataRate = B_9600_BPS;		break;
		case MSG_B_19200_BPS:	DataRate = B_19200_BPS;		break;
		case MSG_B_38400_BPS:	DataRate = B_38400_BPS;		break;
		case MSG_B_57600_BPS:	DataRate = B_57600_BPS;		break;
		case MSG_B_115200_BPS:	DataRate = B_115200_BPS;	break;
		case MSG_B_230400_BPS:	DataRate = B_230400_BPS;	break;
		case MSG_PORT:
			msg->FindString("Port", &str);
			strcpy(PortName, str);
			break;
		case MSG_DISCONNECT:
			connect->SetEnabled(true);
			discon_all();
			connected = false;
			discon->SetEnabled(false);
			break;
		case MSG_CONNECT:
			connect->SetEnabled(false);
			Port->Open(PortName);
			Port->SetDataRate(DataRate);

			thrid = spawn_thread(PsiWrite, "WritePsion", B_NORMAL_PRIORITY, Port);
			if (thrid >= B_NO_ERROR) resume_thread(thrid);

			connected = true;
			discon->SetEnabled(true);
			printf("Connect! %s with %d\n", PortName, DataRate);
			break;
		case MSG_DEBUG:
//			debug = FindView("WriteDebugFile")->Value() == 0 ? 0 : 1;
//			printf("Set debug to %d\n", debug);
			break;
		case MSG_SAVE_PREF:
			SavePreferences();
			break;
		case MSG_NCP_CONN:		// Nachricht von ReadPort, daß eine Verbindung aktiviert wurde!
			msg->FindString("Portname", &str);
			msg->FindInt8("Portnummer", &nr);
			*nam = nr + '0';
			((BTextControl *) FindView(nam))->SetText(str);
			break;
		case MSG_NCP_DISC:		// Nachricht von ReadPort, daß eine Verbindung deaktiviert wurde!
			msg->FindInt8("Portnummer", &nr);
			*nam = nr + '0';
			((BTextControl *) FindView(nam))->SetText("");
			break;
	}
	if (DataRate != 0 && *PortName != '\0' && !discon->IsEnabled()) connect->SetEnabled(true);
}

// Mit dieser Funktion werden alle Einstellungen gesichert!

void PsiNCPWindow::SavePreferences(void) {

	BPath path;
	BFile setting;

	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append("PsionNCP");
	printf("Debug=%d\n", debug);
	setting.SetTo(path.Path(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
	setting.Write(&DataRate, sizeof(DataRate));
	setting.Write(PortName, sizeof(PortName));
	setting.Write(&debug, sizeof(debug));
	setting.Unset();

}

// Mit dieser Funktion werden alle Einstellungen geladen!

void PsiNCPWindow::LoadPreferences(void) {

	BPath path;
	BFile setting;

	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append("PsionNCP");
	
	if (setting.SetTo(path.Path(), B_READ_ONLY) >= B_NO_ERROR) {
		setting.Read(&DataRate, sizeof(DataRate));
		setting.Read(PortName, sizeof(PortName));
		setting.Read(&debug, sizeof(debug));
		setting.Unset();
	}
}

// Was solle passieren, wenn das Fenster geschlossen wird ?

bool PsiNCPWindow::QuitRequested()
{
	if (connected) {
		Minimize(true);
		return(false);
	} else {
		be_app->PostMessage(B_QUIT_REQUESTED);
		return(true);
	}
}

PsiNCPView::PsiNCPView(BRect rect, const char *name, const char *text)
	   	   : BView(rect, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	BTextControl	*bt;
	BTextView		*tv;
	BRect			r;
	int 			i;
	char			chan[3] = "0:";
	char			n[2] = "0";
	
	SetViewColor(216,216,216,0);
	
	r = Bounds();
	r.right -= r.Width() / 2;
	bt = new BTextControl(r, n, chan, "", NULL);
	bt->SetDivider(10);
	tv = bt->TextView();
	tv->SetFontAndColor(be_fixed_font);
	tv->MakeEditable(false);
	AddChild(bt);
	
	for (i = 1; i < NC_MAX_USER; i++) {
		*chan = *n = '0' + i;
		r = bt->Frame();
		bt = new BTextControl(r, n, chan, "", NULL);
		bt->SetDivider(10);
		bt->MoveBy(0, r.Height()+1);
		tv = bt->TextView();
		tv->SetFontAndColor(be_fixed_font);
		tv->MakeEditable(true);
		AddChild(bt);
	}
}

// Dieser View sorgt dafür, daß die Einstellungen auf der Seriellen Schnittstelle
// korrekt versorgt werden und eingestellt werden können.
// Beim Initialisieren wird dafür gesorgt, daß alles gesetzt wird.

SerialView::SerialView(BRect rect, const char *name, BSerialPort *Port, const char *dev, const data_rate dr)
		: BView(rect, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW) 
		
{
	BRect 			r, fr;
	BPopUpMenu		*pop;
	BMenuField		*serial_speed, *serial_device;
	BMenuItem		*mi;
	BMessage		*msg;
	BButton			*save_but;
	BCheckBox		*debug_box;
	char			buffer[B_FILE_NAME_LENGTH];
	font_height		fh;
	float			height, width;
	int32			dev_anz, l;
	
	GetFontHeight(&fh);
	height = fh.ascent + fh.descent;
	SetViewColor(216,216,216,0);
	
	pop = new BPopUpMenu("select port");

	dev_anz = Port->CountDevices();
	for (l = 0; l < dev_anz; l++) {
		Port->GetDeviceName(l, buffer);
		msg = new BMessage(MSG_PORT);
		msg->AddString("Port", buffer);
		mi = new BMenuItem(buffer, msg);
		if (strcmp(buffer, dev) == 0) mi->SetMarked(true);
		pop->AddItem( mi ); 
	}

	r = Bounds();					// Setzen der Positionen!
	r.InsetBy(5,5);
	r.bottom = r.top + height + 5;
	
	serial_device = new BMenuField(r, NULL, "Serial Device:", pop);
	AddChild( serial_device );
	
	pop = new BPopUpMenu("select speed");
	pop->AddItem ( mi = new BMenuItem("9600", new BMessage(MSG_B_9600_BPS) ) );
	if (dr == B_9600_BPS) mi->SetMarked(true);
	pop->AddItem ( mi = new BMenuItem("19200", new BMessage(MSG_B_19200_BPS) ) );
	if (dr == B_19200_BPS) mi->SetMarked(true);
	pop->AddItem ( mi = new BMenuItem("38400", new BMessage(MSG_B_38400_BPS) ) );
	if (dr == B_38400_BPS) mi->SetMarked(true);
	pop->AddItem ( mi = new BMenuItem("57600", new BMessage(MSG_B_57600_BPS) ) );
	if (dr == B_57600_BPS) mi->SetMarked(true);
	pop->AddItem ( mi = new BMenuItem("115200", new BMessage(MSG_B_115200_BPS) ) );
	if (dr == B_115200_BPS) mi->SetMarked(true);
	pop->AddItem ( mi = new BMenuItem("230400", new BMessage(MSG_B_230400_BPS) ) );
	if (dr == B_230400_BPS) mi->SetMarked(true);
	
	serial_speed = new BMenuField(r, NULL, "Serial Speed:", pop);
	serial_speed->MoveBy(0, r.Height()+5);
	AddChild( serial_speed );

	r = serial_speed->Frame();
	debug_box = new BCheckBox(r, "WriteDebugFile", "Write Debugfile", new BMessage(MSG_DEBUG));  
	debug_box->MoveBy(0, r.Height()+5);
	AddChild( debug_box );
	
	fr = Frame();						// Den Save-Butten orientieren wir an den unteren Rand des Fensters
	r.bottom = fr.bottom - 15;
	r.top = r.bottom - (height + 5);
	
	save_but = new BButton(r, NULL, "Save Preferences", new BMessage(MSG_SAVE_PREF), B_FOLLOW_BOTTOM | B_FOLLOW_LEFT_RIGHT);
	save_but->ResizeToPreferred();
	
	width = save_but->Frame().Width();
	save_but->MoveBy((fr.Width() - width) / 2, 0);
	
	AddChild( save_but );

}
